/*
  airbox-dss.js
  Angular service that provides airbox-addon-specific functions
*/

var airboxdss = angular.module('airboxDSS', ['dSS']);


airboxdss.service('airboxDSS',['$q','dss',function($q,dss) {
    var rootpath = "/scripts/wesco";
    var service = this;

    var scenenumbers = [0,5,17,18,19];

    // When reading stringified objects (JSON) from the property tree, string has to be decoded before calling JSON.parse
    function HTMLDecode(string) {
	return string.replace(/&amp;/g,'&').replace(/&gt;/g,'>').replace(/&lt;/g,'<').replace(/&quot;/g,'"');
    }

    // Get list of registered targets for push notifcations. The background script keeps this list up-to-date.
    this.getPushTargetOptions = function() {
	return dss.Property.getString(rootpath+'/Notifications/PushTargetOptions').then(function(result) {
	    return JSON.parse(HTMLDecode(result));
	});
    }

    // Get list of devices that can be selected as exhaust flaps
    /*
      GTIN			Device
      4290046000959	SW-KL200
    */
    this.getExhaustFlapCandidates = function() {
	return dss.Apartment.getDevices().then(function(devices) {
	    var candidates = [];
	    for(var i=0;i<devices.length;i++) {
		if(devices[i].isPresent&&devices[i].GTIN=="4290046000959") {
		    candidates.push({
			'value':devices[i].id,
			'name':(devices[i].name?devices[i].name:devices[i].id)+' ('+devices[i].hwInfo+')',
		    });
		}
	    }
	    return candidates;
	});
    };

    // Set list of push targets (array with device ids) that get notifications
    this.setPushTargets = function(targets) {
	return dss.Property.setString(rootpath+'/Notifications/PushTargets',JSON.stringify(targets));
    };

    // Set dsid of exhaust flap device
    this.setExhaustFlap = function(dsid) {
	return dss.Property.setString(rootpath+'/ExhaustFlap',dsid);
    };

    this.setEmailAddress = function(email) {
	return dss.Property.setString(rootpath+'/Notifications/EmailAddress',email);
    };

    this.getCurrentLevel = function() {
	return dss.Property.getInteger(rootpath+'/Controller/CurrentLevel');
    };

    this.getBusyFlag = function() {
	return dss.Property.getBoolean(rootpath+'/Controller/writingInProgress');
    };

    this.getDayFromVentilationSchedule = function (day){

	var rule={};
	var rules=[];
	return dss.Property.getString(rootpath+'/Controller/VentilationSchedule/'+day).then(function(s){
	    if(s!==undefined && s!==null){
		s = HTMLDecode(s);
		var dict_day=JSON.parse(s);
		for (var h in dict_day){
		    var list_h=dict_day[h];
		    rule={};
		    rule['day']=day;
		    rule['hour']=h;
		    if (list_h[0]!==undefined && list_h[0]!==null){
			rule['Presence']=list_h[0];
		    }
		    if (list_h[1]!==undefined && list_h[1]!==null){
			rule['Absence']=list_h[1];
		    }
		    rules.push(rule);
		}
	    }
	    return rules;
	});
    };

    this.getVentilationSchedule = function() {
	var all_rules=[];

	return dss.Property.getChildren(rootpath+'/Controller/VentilationSchedule/')
	    .then(function(devices) {
		var promises = devices.map(function(device) {
		    return service.getDayFromVentilationSchedule(device.name).then(function(rules){
			for (var rule in rules){
			    all_rules.push(rules[rule]);
			}
		    });
		});
		return $q.all(promises).then(function() {
		    return all_rules;
		});
	    });
    };



    this.setLevel = function(level) {
	return dss.Event.raise('wesco_UI',{
	    'action':'SetAirboxesLevel','level':level,
	});
	//return dss.Apartment.callScene(scenenumbers[level],groupid);
    };

    this.rediscoverAirboxes = function() {
	return dss.Event.raise('wesco_UI',{
	    'action':'RediscoverAirboxes',
	});
    };


    this.refreshPushTargetOptions = function() {
	return dss.Event.raise('wesco_UI',{
	    'action':'RefreshPushTargetOptions',
	});
    };

    this.readRegs = function(dsid,regs) {
	return dss.Event.raise('wesco_UI',{
	    'action':'AirboxRegs','subaction':'readSingle','dSID':dsid,'args':JSON.stringify(regs),
	});
    };

    this.writeRegs = function(dsid,regs) {
	return dss.Event.raise('wesco_UI',{
	    'action':'AirboxRegs','subaction':'writeSingle','dSID':dsid,'args':JSON.stringify(regs),
	});
    };

    this.writeRegs_toAll = function(regs) {
	return dss.Event.raise('wesco_UI',{
	    'action':'AirboxRegs','subaction':'writeAll','args':JSON.stringify(regs),
	});
    };


    this.storeConfigurableParameters = function() {//@Diego
	return dss.Event.raise('wesco_UI',{
	    'action':'StoredConfigurableParameters',
	});
    };

    this.storeExhaustFlap = function() {//@Diego
	return dss.Event.raise('wesco_UI',{
	    'action':'StoredExhaustFlap',
	});
    };

    this.restartAirboxes = function() {//@Diego
	return dss.Event.raise('wesco_UI',{
	    'action':'SetAirboxesLevel','level':'2',
	});
    };

    this.configVentilationSchedule = function(schedule) {//@Diego
	return dss.Event.raise('wesco_UI',{
	    'action':'configVentilationSchedule','schedule':JSON.stringify(schedule),
	});
    };


    this.clearAllAirboxErrors = function() {
	return dss.Event.raise('wesco_UI',{
	    'action':'ClearAllAirboxErrors',
	});
    };

    this.clearAllAirboxErrors = function() {
	return dss.Event.raise('wesco_UI',{
	    'action':'ClearAllAirboxErrors',
	});
    };

    this.clearAirboxError = function(dSUID) {
	return dss.Event.raise('wesco_UI',{
	    'action':'ClearAirboxError','dSUID':dSUID,
	});
    };

    this.getErrorLog = function() {
	return dss.getLogfile('/dss/js/wesco.log');
    };

    this.getConfig = function() {
	var result = {};
	var promises = [];
	promises.push(dss.Property.getString(rootpath+'/Notifications/PushTargets').then(function(value) {
	    result['PushTargets'] = JSON.parse(HTMLDecode(value));
	}));
	promises.push(dss.Property.getString(rootpath+'/Notifications/EmailAddress').then(function(value) {
	    result['EmailAddress'] = value;
	}));
	promises.push(dss.Property.getString(rootpath+'/ExhaustFlap').then(function(value) {
	    result['ExhaustFlap'] = value;
	}));
	promises.push(dss.Property.getInteger(rootpath+'/Controller/ClusterID').then(function(value) {
	    result['ClusterID'] = value;
	}));
	return $q.all(promises).then(function() {
	    return result;
	});
    };

    this.getAirboxConfig = function() {
	var result = {};
	var promises = [];
	promises.push(dss.Property.getInteger(rootpath+'/Airboxes/FilterReplacementInterval').then(function(value) {
	    result['FilterReplacementInterval'] = value;
	}));
	return $q.all(promises).then(function() {
	    return result;
	});
    };

    // Get info for a single airbox
    this.getAirbox = function(deviceid) {
	var result = {};
	var promises = [];

	result['dSUID']=deviceid; // added variable dsuid (which is different than the dsid)!

	promises.push(dss.Property.getString(rootpath+'/Airboxes/Devices/'+deviceid+'/dSID').then(function(value) {
	    result['dSID'] = value;
	    return dss.Device.getName(result['dSID']).then(function(name) {
		result['Description'] = name;
	    });
	}));
	promises.push(dss.Property.getString(rootpath+'/Airboxes/Devices/'+deviceid+'/errorCode').then(function(value) {
	    result['errorCode'] = value;
	}));
	promises.push(dss.Property.getString(rootpath+'/Airboxes/Devices/'+deviceid+'/errorTime').then(function(value) {
	    result['errorTime'] = value;
	}));
	promises.push(dss.Property.getInteger(rootpath+'/Airboxes/Devices/'+deviceid+'/errorCounter').then(function(value) {
	    result['errorTime'] = value;
	}));
	return $q.all(promises).then(function() {
	    return result;
	});
    };

    // Get info for all configured airboxes
    this.getAirboxes = function() {
	var listReady = dss.Property.getChildren(rootpath+'/Airboxes/Devices')
	    .then(function(devices) {
		var devInfoPromises = devices.map(function(device) {
		    return service.getAirbox(device.name);
		});
		return $q.all(devInfoPromises);
	    });
	return listReady;
    };

    this.parseVarFile = function(contents) {
	var result = [];
	var lines = contents.split(/\n/);
	for(var i=0;i<lines.length;i++) {
	    if (!/\S/.test(lines[i])) {
		continue;
	    }
	    if (/#/.test(lines[i])) { // skip commentaren
		continue;
	    }

	    var data=lines[i].split(/\t/);
	    result.push({
		'reg':data[0],
		'value':data[1],
	    });
	}
	return result;
    }

}]);
